X-Git-Url: https://www.chiark.greenend.org.uk/ucgi/~mdw/git/disorder/blobdiff_plain/14ad73b94d56fb8558eabfef431c34e12de00478..ad0c0f361be0:/lib/log.c?ds=inline
diff --git a/lib/log.c b/lib/log.c
index 0ebda63..82ba377 100644
--- a/lib/log.c
+++ b/lib/log.c
@@ -1,48 +1,45 @@
/*
* This file is part of DisOrder.
- * Copyright (C) 2004, 2005, 2006 Richard Kettlewell
+ * Copyright (C) 2004-2008 Richard Kettlewell
*
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * General Public License for more details.
- *
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
* You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * along with this program. If not, see .
*/
/** @file lib/log.c @brief Errors and logging
*
* All messages are initially emitted by one of the four functions below.
* debug() is generally invoked via D() so that mostly you just do a test
* rather than a complete subroutine call.
*
* Messages are dispatched via @ref log_default. This defaults to @ref
* log_stderr. daemonize() will turn off @ref log_stderr and use @ref
* log_syslog instead.
*
* fatal() will call exitfn() with a nonzero status. The default value is
* exit(), but it should be set to _exit() anywhere but the 'main line' of the
* program, to guarantee that exit() gets called at most once.
*/
#define NO_MEMORY_ALLOCATION
/* because the memory allocation functions report errors */
-#include
+#include "common.h"
-#include
-#include
-#include
#include
#include
#include
+#include
#include "log.h"
#include "disorder.h"
@@ -71,6 +68,9 @@ const char *progname;
/** @brief Filename for debug messages */
const char *debug_filename;
+/** @brief Set to include timestamps in log messages */
+int logdate;
+
/** @brief Line number for debug messages */
int debug_lineno;
@@ -92,8 +92,10 @@ static void format(char buffer[], size_t bufsize, const char *fmt, va_list ap) {
int ch;
size_t n = 0;
- if(byte_vsnprintf(t, sizeof t, fmt, ap) < 0)
- strcpy(t, "[byte_vsnprintf failed]");
+ if(byte_vsnprintf(t, sizeof t, fmt, ap) < 0) {
+ strcpy(t, "[byte_vsnprintf failed: ");
+ strncat(t, fmt, sizeof t - strlen(t) - 1);
+ }
p = t;
while((ch = (unsigned char)*p++)) {
if(ch >= ' ' && ch <= 126) {
@@ -111,6 +113,8 @@ static void format(char buffer[], size_t bufsize, const char *fmt, va_list ap) {
}
/** @brief Log to a file
+ * @param pri Message priority (as per syslog)
+ * @param msg Messagge to log
* @param user The @c FILE @c * to log to or NULL for @c stderr
*/
static void logfp(int pri, const char *msg, void *user) {
@@ -120,7 +124,15 @@ static void logfp(int pri, const char *msg, void *user) {
* sanely */
const char *p;
- if(progname)
+ if(logdate) {
+ char timebuf[64];
+ struct tm *tm;
+ gettimeofday(&tv, 0);
+ tm = localtime(&tv.tv_sec);
+ strftime(timebuf, sizeof timebuf, "%Y-%m-%d %H:%M:%S %Z", tm);
+ fprintf(fp, "%s: ", timebuf);
+ }
+ if(progname)
fprintf(fp, "%s: ", progname);
if(pri <= LOG_ERR)
fputs("ERROR: ", fp);
@@ -177,7 +189,10 @@ static void logger(int pri, const char *fmt, ...) {
}
/** @brief Format and log a message
+ * @param pri Message priority (as per syslog)
+ * @param fmt Format string
* @param errno_value Errno value to include as a string, or 0
+ * @param ap Argument list
*/
void elog(int pri, int errno_value, const char *fmt, va_list ap) {
char buffer[1024];
@@ -192,15 +207,41 @@ void elog(int pri, int errno_value, const char *fmt, va_list ap) {
}
}
-#define disorder_fatal fatal
-#define disorder_error error
-#define disorder_info info
+/** @brief Log an error and quit
+ *
+ * If @c ${DISORDER_FATAL_ABORT} is defined (as anything) then the process
+ * is aborted, so you can get a backtrace.
+ */
+void disorder_fatal(int errno_value, const char *msg, ...) {
+ va_list ap;
+
+ va_start(ap, msg);
+ elog(LOG_CRIT, errno_value, msg, ap);
+ va_end(ap);
+ if(getenv("DISORDER_FATAL_ABORT")) abort();
+ exitfn(EXIT_FAILURE);
+}
-/* shared implementation of vararg functions */
-#include "log-impl.h"
+/** @brief Log an error */
+void disorder_error(int errno_value, const char *msg, ...) {
+ va_list ap;
+
+ va_start(ap, msg);
+ elog(LOG_ERR, errno_value, msg, ap);
+ va_end(ap);
+}
+
+/** @brief Log an informational message */
+void disorder_info(const char *msg, ...) {
+ va_list ap;
+
+ va_start(ap, msg);
+ elog(LOG_INFO, 0, msg, ap);
+ va_end(ap);
+}
/** @brief Log a debug message */
-void debug(const char *msg, ...) {
+void disorder_debug(const char *msg, ...) {
va_list ap;
va_start(ap, msg);
@@ -1,48 +1,45 @@
/** @file lib/log.c @brief Errors and logging
*
- * All messages are initially emitted by one of the four functions below.
- * debug() is generally invoked via D() so that mostly you just do a test
- * rather than a complete subroutine call.
+ * All messages are initially emitted by one of the four functions
+ * below. disorder_debug() is generally invoked via D() so that
+ * mostly you just do a test rather than a complete subroutine call.
*
* Messages are dispatched via @ref log_default. This defaults to @ref
@@ -1,48 +1,45 @@
* log_syslog instead.
*
- * fatal() will call exitfn() with a nonzero status. The default value is
- * exit(), but it should be set to _exit() anywhere but the 'main line' of the
- * program, to guarantee that exit() gets called at most once.
+ * disorder_fatal() will call exitfn() with a nonzero status. The
+ * default value is exit(), but it should be set to _exit() anywhere
+ * but the 'main line' of the program, to guarantee that exit() gets
+ * called at most once.
*/
#define NO_MEMORY_ALLOCATION